/*
 * File:   main.c
 * Author: yts
 *
 * Created on 2017/10/20, 18:14
 */

#include <xc.h>
#include <dsp.h>

#define LOG2N 6
#define FFT_SIZE 64 //=2^6


//  Place the twiddle factors in a PSV section (or X-memory)
//  Only FFT_SIZE/2 twiddle factors are needed for an FFT
const fractcomplex twiddleFactors[FFT_SIZE/2] __attribute__ ((space(auto_psv), aligned (FFT_SIZE/2 * 2 * 2))) ={
    0x7FFF, 0x0000, 0x7F62, 0xF374, 0x7D8A, 0xE707, 0x7A7D, 0xDAD8,
    0x7642, 0xCF04, 0x70E3, 0xC3A9, 0x6A6E, 0xB8E3, 0x62F2, 0xAECC,
    0x5A82, 0xA57E, 0x5134, 0x9D0E, 0x471D, 0x9592, 0x3C57, 0x8F1D,
    0x30FC, 0x89BE, 0x2528, 0x8583, 0x18F9, 0x8276, 0x0C8C, 0x809E,
    0x0000, 0x8000, 0xF374, 0x809E, 0xE707, 0x8276, 0xDAD8, 0x8583,
    0xCF04, 0x89BE, 0xC3A9, 0x8F1D, 0xB8E3, 0x9592, 0xAECC, 0x9D0E,
    0xA57D, 0xA57D, 0x9D0E, 0xAECC, 0x9592, 0xB8E3, 0x8F1D, 0xC3A9,
    0x89BE, 0xCF04, 0x8583, 0xDAD8, 0x8276, 0xE707, 0x809E, 0xF374
};


// Input Array containing sinusoidal wave signal of frequency 1KHz sampled at 8 KHz
fractional sin1k[FFT_SIZE]={
    0x0000,0x5A81,0x7FFF,0x5A81,0x0000,0xA57F,0x8001,0xA57F,0x0000,0x5A81,
    0x7FFF,0x5A81,0x0000,0xA57F,0x8001,0xA57F,0x0000,0x5A81,0x7FFF,0x5A81,
    0x0000,0xA57F,0x8001,0xA57F,0x0000,0x5A81,0x7FFF,0x5A81,0x0000,0xA57F,
    0x8001,0xA57F,0x0000,0x5A81,0x7FFF,0x5A81,0x0000,0xA57F,0x8001,0xA57F,
    0x0000,0x5A81,0x7FFF,0x5A81,0x0000,0xA57F,0x8001,0xA57F,0x0000,0x5A81,
    0x7FFF,0x5A81,0x0000,0xA57F,0x8001,0xA57F,0x0000,0x5A81,0x7FFF,0x5A81,
    0x0000,0xA57F,0x8001,0xA57F
};

// Place fftBuffer in Y-memory aligned to the size the array in bytes
fractcomplex fftBuffer[ FFT_SIZE ] __attribute__ ((section (".ybss, bss, ymemory"), aligned (FFT_SIZE * 2 * 2)));

//  Hamming window
fractional window[FFT_SIZE];

// Power Spectrum
fractional powerSpec[ FFT_SIZE/2 ];   

int main(void) {
    int i;

//  uncomment to get the twiddle factors
//    fractcomplex twiddleFactors1[ FFT_SIZE/2];
//    TwidFactorInit(LOG2N, (fractcomplex *)twiddleFactors1, 0);
    
//  uncommnet to get the Hamming window
//    HammingInit (FFT_SIZE, window);    

//  uncomment if the window function is preferred
//    VectorWindow(FFT_SIZE, sin1k, sin1k, window);  


    // real part: data , imaginary part: 0
    for(i = 0; i < FFT_SIZE; i++) {
        fftBuffer[i].real = sin1k[i]/2;  // data should be in the range from -0.5 to +0.5
        fftBuffer[i].imag = 0;
    }

    // execute FFT
    FFTComplexIP(LOG2N, fftBuffer, (fractcomplex*)__builtin_psvoffset (&twiddleFactors[0]),
                (int) __builtin_psvpage(&twiddleFactors[0]));

    // bit reversal 
    BitReverseComplex(LOG2N, fftBuffer ); 

    // absolute values below 4 kHz
    SquareMagnitudeCplx( FFT_SIZE/2 , fftBuffer, powerSpec ); 
    
    while(1);

    return(0);
}
